@@ -9,6 +9,10 @@ Copyright (C) 2010 Nick Leppänen Larsson <frals@frals.se>
 """
 import os
 import sys
+import gettext
+import codecs
+import os.path
+import shutil
 
 import gtk
 import hildon
@@ -24,6 +28,11 @@ import contacts as ContactH
 import logging
 log = logging.getLogger('fmms.%s' % __name__)
 
+_ = gettext.gettext
+gettext.bindtextdomain('fmms','/opt/fmms/share/locale/')
+gettext.textdomain('fmms')
+
+
 class fMMS_Viewer(hildon.Program):
 
 	def __init__(self, fname, standalone=False, spawner=None):
@@ -38,7 +47,7 @@ class fMMS_Viewer(hildon.Program):
 		self.spawner = spawner
 		
 		self.window = hildon.StackableWindow()
-		self.window.set_title("Showing MMS")
+		self.window.set_title("MMS")
 		self.window.connect("delete_event", self.quit)
 		
 		vbox = gtk.VBox()
@@ -50,7 +59,11 @@ class fMMS_Viewer(hildon.Program):
 		self._parse_mms(fname, vbox)
 
 		pan.add_with_viewport(vbox)
-		self.window.add(pan)
+		
+		align = gtk.Alignment(1, 1, 1, 1)
+		align.set_padding(2, 2, 10, 10)
+		align.add(pan)
+		self.window.add(align)
 
 		if not self.cont.is_mms_read(fname) and self._direction == fMMSController.MSG_DIRECTION_IN:
 			self.cont.mark_mms_read(fname)
@@ -76,34 +89,46 @@ class fMMS_Viewer(hildon.Program):
 		""" create app menu for mms viewing window """
 		menu = hildon.AppMenu()
 		
+		self.headerstxt = _("Headers")
 		headers = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		headers.set_label("Headers")
+		headers.set_label(self.headerstxt)
 		headers.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.replytxt = gettext.ldgettext('skype-ui', 'skype_ti_incoming_call_options')
 		reply = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		reply.set_label("Reply")
+		reply.set_label(self.replytxt)
 		reply.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.replysmstxt = "%s (%s)" % (gettext.ldgettext('skype-ui', 'skype_ti_incoming_call_options'), "SMS")
 		replysms = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		replysms.set_label("Reply via SMS")
+		replysms.set_label(self.replysmstxt)
 		replysms.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.forwardtxt = gettext.ldgettext('rtcom-messaging-ui', 'messaging_fi_forward')
 		forward = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		forward.set_label("Forward")
+		forward.set_label(self.forwardtxt)
 		forward.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.copytxt = "%s" % gettext.ldgettext('rtcom-messaging-ui', 'messaging_fi_copy')
 		copyb = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		copyb.set_label("Copy text")
+		copyb.set_label(self.copytxt)
 		copyb.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.deletetxt = gettext.ldgettext('hildon-libs', 'wdgt_bd_delete')
 		delete = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		delete.set_label("Delete")
+		delete.set_label(self.deletetxt)
 		delete.connect('clicked', self.mms_menu_button_clicked, fname)
 		
+		self.saveattachments = gettext.ldgettext('modest', 'mcen_me_viewer_save_attachments')
+		saveattach = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		saveattach.set_label(self.saveattachments)
+		saveattach.connect('clicked', self.mms_menu_button_clicked, fname)
+		
 		menu.append(reply)
 		menu.append(replysms)
-		menu.append(forward)
+		menu.append(saveattach)
 		menu.append(copyb)
+		menu.append(forward)
 		menu.append(headers)
 		menu.append(delete)
 		menu.show_all()
@@ -112,10 +137,11 @@ class fMMS_Viewer(hildon.Program):
 	
 	def delete_dialog(self, filename):
 		dialog = gtk.Dialog()
-		dialog.set_title("Confirm")
+		confirmtxt = gettext.ldgettext('rtcom-messaging-ui', "messaging_fi_delete_1_sms")
+		dialog.set_title(confirmtxt)
 		dialog.add_button(gtk.STOCK_YES, 1)
 		dialog.add_button(gtk.STOCK_NO, 0)
-		label = gtk.Label("Are you sure you want to delete the message?")
+		label = gtk.Label(confirmtxt)
 		dialog.vbox.add(label)
 		dialog.show_all()
 		ret = dialog.run()
@@ -139,43 +165,46 @@ class fMMS_Viewer(hildon.Program):
 			self.window.destroy()
 		except Exception, e:
 			log.exception("%s %s", type(e), e)
-			banner = hildon.hildon_banner_show_information(self.window, "", "Failed to delete message.")
+			banner = hildon.hildon_banner_show_information(self.window, "", \
+					gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed"))
 
 	def mms_menu_button_clicked(self, button, fname):
 		""" actions for mms menu """
 		buttontext = button.get_label()
-		if buttontext == "Headers":
+		if buttontext == self.headerstxt:
 			ret = self.create_headers_dialog(fname)
-		elif buttontext == "Reply":
+		elif buttontext == self.replytxt:
 			number = self.cont.get_replyuri_from_transid(fname)
 			fMMSSenderUI.fMMS_SenderUI(spawner=self.window, tonumber=number).run()
-		elif buttontext == "Reply via SMS":
+		elif buttontext == self.replysmstxt:
 			number = self.cont.get_replyuri_from_transid(fname)
 			if "@" in number:
 				note = osso.SystemNote(self.osso_c)
-				note.system_note_dialog("Reply address does not look like a phone number, sorry." , 'notice')
+				note.system_note_dialog(gettext.ldgettext('rtcom-messaging-ui', "messaging_fi_smsc_invalid_chars") , 'notice')
 			else:
 				rpc = osso.Rpc(self.osso_c)
 				nr = "sms:%s" % str(number)
 				args = (nr, "")
 				rpc.rpc_run('com.nokia.MessagingUI', '/com/nokia/MessagingUI', 'com.nokia.MessagingUI', 'messaging_ui_interface_start_sms', args, True, True)
-		elif buttontext == "Forward":
+		elif buttontext == self.forwardtxt:
 			tbuffer = self.textview.get_buffer()
 			msg = tbuffer.get_text(tbuffer.get_start_iter(), tbuffer.get_end_iter())
 			fn = self.attachment
-			fMMSSenderUI.fMMS_SenderUI(spawner=self.window, withfile=fn, message=msg)
-		elif buttontext == "Delete":
+			fMMSSenderUI.fMMS_SenderUI(spawner=self.window, withfile=fn, message=msg, forward=True)
+		elif buttontext == self.deletetxt:
 			self.delete_dialog(fname)
-		elif buttontext == "Copy text":
+		elif buttontext == self.copytxt:
 			clip = gtk.Clipboard(display=gtk.gdk.display_get_default(), selection="CLIPBOARD")
 			tbuffer = self.textview.get_buffer()
 			msg = tbuffer.get_text(tbuffer.get_start_iter(), tbuffer.get_end_iter())
 			clip.set_text(msg, -1)
+		elif buttontext == self.saveattachments:
+			self.save_attachments(fname)
 			
 	def create_headers_dialog(self, fname):
 		""" show headers in a dialog """
 		dialog = gtk.Dialog()
-		dialog.set_title("Headers")
+		dialog.set_title(self.headerstxt)
 		
 		dialogVBox = gtk.VBox()
 		
@@ -213,7 +242,8 @@ class fMMS_Viewer(hildon.Program):
 		self.force_ui_update()
 		
 		if not self.cont.is_fetched_push_by_transid(filename):
-			banner = hildon.hildon_banner_show_information(self.window, "", "Trying to download MMS...")
+			msgstr = gettext.ldgettext('hildon-application-manager', "ai_nw_downloading") % "MMS"
+			banner = hildon.hildon_banner_show_information(self.window, "", msgstr)
 			self.force_ui_update()
 			self.cont.get_mms_from_push(filename)
 			self.cont.mark_mms_read(filename)
@@ -224,10 +254,12 @@ class fMMS_Viewer(hildon.Program):
 		topbox = gtk.HBox()
 		
 		if self._direction == fMMSController.MSG_DIRECTION_IN:
-			label = gtk.Label('<span foreground="#666666">From</span>')
+			label = gtk.Label('<span foreground="#666666">%s</span>' \
+					  % gettext.ldgettext('modest', 'mail_va_from'))
 			sender = headerlist.get('From', "0").replace("/TYPE=PLMN", "")
 		else:
-			label = gtk.Label('<span foreground="#666666">To</span>')
+			label = gtk.Label('<span foreground="#666666">%s</span>' \
+					  % gettext.ldgettext('rtcom-messaging-ui', 'messaging_fi_new_sms_to'))
 			sender = headerlist['To'].replace("/TYPE=PLMN", "")
 		
 		label.set_use_markup(True)
@@ -251,9 +283,9 @@ class fMMS_Viewer(hildon.Program):
 		timelabel.set_use_markup(True)
 		timelabel.set_alignment(1, 0.5)
 
-		topbox.pack_start(label, False, False, 20)
-		topbox.pack_start(namelabel, True, True, 0)
-		topbox.pack_end(timelabel, False, False, 10)
+		topbox.pack_start(label, False, False, 0)
+		topbox.pack_start(namelabel, True, True, 10)
+		topbox.pack_end(timelabel, False, False, 0)
 		
 		container.pack_start(topbox, False, False, 5)
 		sep = gtk.HSeparator()
@@ -265,7 +297,7 @@ class fMMS_Viewer(hildon.Program):
 		self.textview.set_editable(False)
 		self.textview.set_cursor_visible(False)
 		self.textview.set_wrap_mode(gtk.WRAP_WORD)
-		self.textview.set_justification(gtk.JUSTIFY_CENTER)
+		self.textview.set_justification(gtk.JUSTIFY_LEFT)
 		textbuffer = gtk.TextBuffer()
 		direction = self.cont.get_direction_mms(filename)
 		
@@ -293,7 +325,7 @@ class fMMS_Viewer(hildon.Program):
 			except Exception, e:
 				filetype = None
 				log.exception("%s %s", type(e), e)
-			
+
 			if isImage or ext == ".wbmp":
 				""" insert the image in an eventbox so we can get signals """
 				ebox = gtk.EventBox()
@@ -311,8 +343,13 @@ class fMMS_Viewer(hildon.Program):
 				container.add(ebox)
 				self.attachment = fnpath
 			elif isText or ext.startswith(".txt"):
-				fp = open(path + "/" + fname, 'r')
-				contents = fp.read()
+				try:
+					fp = codecs.open(path + "/" + fname, 'r', 'utf-8')
+					contents = fp.read()
+				except:
+					fp = codecs.open(path + "/" + fname, 'r', 'latin-1')
+					contents = fp.read()
+				
 				fp.close()
 				textbuffer.insert(textbuffer.get_end_iter(), contents)
 			elif name != "message" and name != "headers" and not ext.startswith(".smil") and filetype != "application/smil":
@@ -356,12 +393,86 @@ class fMMS_Viewer(hildon.Program):
 		menu = gtk.Menu()
 		menu.set_property("name", "hildon-context-sensitive-menu")
 
-		openItem = gtk.MenuItem("Open")
+		openItem = gtk.MenuItem(gettext.ldgettext('hildon-fm', 'ckdg_ti_open_file'))
 		menu.append(openItem)
 		openItem.connect("activate", self.mms_img_clicked, data)
 		openItem.show()
 		menu.show_all()
 		return menu
+		
+	def save_attachments(self, filename):
+		src = self.cont.get_filepath_for_mms_transid(filename)
+		attachments = self.cont.get_mms_attachments(filename)
+		selector = hildon.TouchSelector(text=True)
+		for fn in attachments:
+			selector.append_text(fn)
+
+		selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
+		selector.unselect_all(0)
+		pan = hildon.PannableArea()
+		pan.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
+		pan.add_with_viewport(selector)
+		pan.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
+
+		diag = gtk.Dialog()
+		diag.set_title(self.saveattachments)
+		diag.set_transient_for(self.window)
+		diag.vbox.add(pan)
+		diag.set_size_request(-1, 320)
+		diag.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_APPLY)
+		diag.show_all()
+		ret = diag.run()
+		selection = selector.get_current_text()
+		sel_count = len(selector.get_selected_rows(0))
+		diag.destroy()
+		if ret == gtk.RESPONSE_APPLY and sel_count > 0:
+			self.choose_your_destiny(src, attachments, selection, sel_count)
+		
+	def choose_your_destiny(self, src, attachments, current_text, current_count):
+		current_text = current_text.replace('(', '').replace(')','')
+		fcd = hildon.FileChooserDialog(self.window, action=gtk.FILE_CHOOSER_ACTION_SAVE)
+		fcd.set_transient_for(self.window)
+		if os.path.isdir("/home/user/MyDocs/"):
+			fcd.set_current_folder("/home/user/MyDocs/")
+		fcd.set_default_response(gtk.RESPONSE_OK)
+		if current_count == 1:
+			fn = current_text
+			fcd.set_current_name(fn)
+			fcd.set_title(gettext.ldgettext('hildon-fm', 'sfil_ti_save_file'))
+		else:
+			strng = 'sfil_va_number_of_objects_attachment'
+			txt = gettext.ldngettext('hildon-fm',strng, strng, current_count) % current_count
+			fcd.set_property('save-multiple', txt)
+			fcd.set_title(gettext.ldgettext('hildon-fm', 'sfil_ti_save_objects_files'))
+		ret = fcd.run()
+		cur_folder = fcd.get_current_folder_uri()
+		cur_uri = fcd.get_uri()
+		fcd.destroy()
+		if ret == gtk.RESPONSE_OK:
+			dst = cur_uri.replace('file://', '')
+			if current_count == 1:
+				# dst is fullpath + fn here
+				srcfn = "%s/%s" % (src, fn)
+				self.copy_file(srcfn, dst)
+			else:
+				# dst is only fullpath here
+				files = current_text.split(',')
+				for fn in files:
+					dstfn = "%s/%s" % (dst, fn)
+					srcfn = "%s/%s" % (src, fn)
+					self.copy_file(srcfn, dstfn)
+
+
+	def copy_file(self, src, dst):
+		if not os.path.isfile(dst):
+			log.info("copying %s -> %s" % (src, dst))
+			shutil.copy(src, dst)
+			banner = hildon.hildon_banner_show_information(self.window, "", \
+					gettext.ldgettext('hildon-libs', "wdgt_bd_done"))
+		else:
+			log.info("copy failed: file exists (%s -> %s)" % (src, dst))
+			banner = hildon.hildon_banner_show_information(self.window, "", \
+					gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed"))
 
 	def run(self):
 		self.window.show_all()
